#!/usr/bin/env python
"""pikpak a python/gtk front end for ipkg"""

import gtk,GtkExtra,os,sys
from glob import glob
from string import split, strip, find

## Constants ##

VERSION = 0.12

## All Possible field names (hopefully)

FLD_LU = { 'Package':0, 'Essential':1, 'Priority':2, 'Section':3, 'Installed-Size':4,
           'Debian-Maintainer':5, 'Maintainer':6, 'Architecture':7, 'Version':8, 'Replaces':9,
           'Pre-Depends':10, 'Conflicts':11, 'Filename':12, 'Size':13, 'MD5sum':14, 'Description':15,
           'Depends':16, 'Origin':17, 'Bugs':18, 'Provides':19, 'Suggests':20 }


## Edit this to change what fields are displayed and there widths

COLS = { 'Package':100, 'Version':50, 'Size':50, 'Section':50, 'Priority':50 }

##  Location of ipkg script

##IPKG_BIN = '/home/mallum/Projects/familiar/dist/ipkg/sh/ipkg'
IPKG_BIN = 'ipkg'

## Constants for menu / package actions 

IPKG_UPDATE      = 3
IPKG_INSTALL     = 1
IPKG_REMOVE      = 2
IPKG_UPGRADE     = 4
IPKG_UPGRADE_ALL = 5

## Constants for defining view types

V_ALL        = 0
V_INST       = 1
V_NOTINST    = 2

## Colors

UNINSTALLED_COLOR = 'dark red'
TO_UPGRADE_COLOR  = 'red'

## Interactive

INTERACTIVE = 1

class Pkgs:
    """Class for managing packages and states"""
    def __init__(self):
        ## Data structs for package info


        self.view    = []

        self.viewBy  = V_ALL
        self.sorted_by = 'Package'        

        ## init all 
        self.get_available()
        self.get_installed()
        self.update_view()

    def sort(self,bywhat,no_reverse=1):              ## TODO: Required->Important->Standard->Optional->Extra
        if FLD_LU.has_key(bywhat):
            if self.sorted_by == bywhat and no_reverse:
                self.view.reverse()
            else:
                self.view.sort(lambda s1,s2, bywhat=bywhat:
                               cmp(s1[FLD_LU[bywhat]] ,
                                   s2[FLD_LU[bywhat]] ) )
                self.sorted_by= bywhat


    def find(self, txt, lastFindIndex = 0):
        for p in range(lastFindIndex, len(self.view)):
            for part in self.view[p]:
                if find(str(part), txt) != -1:                    
                    return p
        return -1    

    def update_view(self, viewby = V_ALL):
        self.viewBy = viewby
        if self.viewBy == V_ALL:
            self.view = self.available
            return 
        
        self.view = []
        for p in self.available:
            if self.viewBy == V_INST and self.is_installed(p[FLD_LU['Package']]):
                self.view.append(p)
            elif self.viewBy == V_NOTINST and not self.is_installed(p[FLD_LU['Package']]):
                self.view.append(p)
        self.sort(self.sorted_by,0)

    def get_field(self,row_num,col_name):
        if FLD_LU.has_key[col_name]:
            return self.view[row_num][FLD_LU[col_name]]
        else:
            return 0

    def is_installed(self,pkgname):
        return self.installed.has_key(pkgname)

    def get_installed_version(self,pkgname):
        if self.installed.has_key(pkgname):
            return self.installed[pkgname]
        else:
            return 0

    def get_available(self):                      ## Passes the output of ipkg info
        try:                                      ## into a 2D indexed array
            f = os.popen(IPKG_BIN + " info")
        except:
            print "ERROR :: Couldn't execute ipkg info! "
            print "         Check ipkg is in your path  "
            sys.exit()
            
        self.available = []
            
        pkg = [''] * 25                
        line = f.readline()
        while line:
            if line == "\n":                       ## Its a new Package entry
                if pkg[FLD_LU['Package']]:         ## Check its not just a blank line
                    self.available.append(pkg)
                pkg = [''] * 25
            else:
                parts = split(line[:-1],":")       ## remove newline
                if ':' in line and FLD_LU.has_key(parts[0]):
                    if parts[0] == 'Size':         ## Make size field more readable
                        pkg[FLD_LU[parts[0]]] = int(strip(parts[1]))/1024
                    else:
                        pkg[FLD_LU[parts[0]]] = strip(parts[1])                    
                else:                              ## Build description field
                    if pkg[FLD_LU["Description"]] == '.\n':
                        pkg[FLD_LU["Description"]] = pkg[FLD_LU["Description"]] + '\n\n'
                    else:
                        pkg[FLD_LU["Description"]] = pkg[FLD_LU["Description"]] + parts[0]

            line = f.readline()
	f.close()

    def get_installed(self):                      ## Parses 'ipkg status' into a dictionary
        self.installed = {}                       ## containing package -> version
        try:                                       
            f = os.popen(IPKG_BIN + " status","r")
        except:
            print "ERROR :: Couldn't execute ipkg status ! "
            print "         Check ipkg is in your path  "
            sys.exit()

        line = f.readline()
        while line:
            if line != '\n':
                p = split(line,":")
                if p[0] == 'Package':
                    current_pkg = strip(p[1])
                if p[0] == "Version":
                    self.installed[current_pkg] = strip(p[1])
            line = f.readline()                   
        f.close                                   


class MsgBox(gtk.GtkDialog):                      ## simple sketchy dialog class
                                                  ## TODO: make more sense / modular !
    def __init__(self,master,lbl_text='',type=1,functions=None):
        gtk.GtkDialog.__init__(self)
        self.connect("delete_event", self.delete_event)
        self.top = master
        self.set_usize(180,100)
        self.set_uposition(20,80)
        lbl = gtk.GtkLabel(lbl_text)
        lbl.show()
        lbl.set_justify(gtk.JUSTIFY_CENTER)
        lbl.set_line_wrap(gtk.TRUE)
        self.vbox.pack_start(lbl)
        if type == 1:                             ## ok box
            self.okay = gtk.GtkButton('Okay')
            self.okay.show()
            self.action_area.pack_start(self.okay,expand=gtk.FALSE)
            if functions:
                self.okay.connect('clicked',functions[0])
            else:
                self.okay.connect('clicked',self.close_me)
        elif type == 2:                           ## find box
            self.findbutt = gtk.GtkButton('Find')
            self.findbutt.show()
            self.cancelbutt = gtk.GtkButton('Cancel')
            self.cancelbutt.show()
            self.action_area.pack_start(self.findbutt,expand=gtk.FALSE)
            self.action_area.pack_start(self.cancelbutt,expand=gtk.FALSE)
            self.entry = gtk.GtkEntry()
            self.entry.show()
            self.vbox.pack_start(self.entry)
            self.findbutt.connect('clicked',self.findCB)
            self.cancelbutt.connect('clicked',self.close_me)
             
        self.vbox.show()
        self.action_area.show()
        self.set_modal(gtk.TRUE)

    def findCB(self,btn):
        self.hide()
        self.top.searchStr = self.entry.get_text()
        self.top.lastFindIndex = 0
        self.top.doFind()

    def delete_event(win, event=None):
        self.hide()
        return gtk.TRUE

    def close_me(self,btn):
        self.hide()
        del self
        

class AppWindow(gtk.GtkWindow):         # Usual Base
    def __init__(self, title, width, height):
        gtk.GtkWindow.__init__(self)
        if title:
            self.set_title(title)
        self.set_usize(width,height)
        self.connect("delete_event", self.quit)

    def quit(self, *args):
        gtk.mainquit()

class Pikpak:

    def __init__(self):
        self.pkgs = Pkgs()                ## init the pkgs object
        if gtk.screen_width() > 300:      ## Fit the display
            self.top = AppWindow("Pikpak", 320, 200)
        else:
            self.top = AppWindow("Pikpak", 240, 300)

        self.cmap = self.top.get_colormap() 

        self.vpane = gtk.GtkVPaned();   # to hold everything

        self.box = gtk.GtkVBox()
        self.top.add(self.box)
        self.box.show()
        self.init_menu()        

        self.box.pack_end(self.vpane, fill=gtk.TRUE, expand=gtk.TRUE)
        self.vpane.show()
        
        self.init_pkg_clist()
        self.init_info()

        self.selected = ""
        self.lastFindIndex = 0
        self.searchStr     = ''

        self.checkItemCalled = 0        ## Needed for ugly menu callback
        self.itemf.get_widget('/View/All' ).set_active(gtk.TRUE)
        
        self.update_pkg_list()

    def doFind(self):
        foundrow = self.pkgs.find(self.searchStr, self.lastFindIndex)
        if foundrow != -1:
            self.lastFindIndex = foundrow + 1
            self.clist.moveto( foundrow, 0, 0.5, 0.5 )
            self.clist.select_row( foundrow, 0)
        else:
            MsgBox(self,"%s not found" % self.searchStr ).show()
            self.lastFindIndex = 0

    def init_menu(self):
        ag = gtk.GtkAccelGroup()
        self.itemf = gtk.GtkItemFactory(gtk.GtkMenuBar, "<main>", ag)
        self.top.add_accel_group(ag)
        self.itemf.create_items([
            ('/File',             None, None, 0, '<Branch>'),
            ('/File/Update',      None, self.packageCB, IPKG_UPDATE, ''),
            ('/File/Upgrade (all)', None, self.packageCB, IPKG_UPGRADE_ALL, ''),
            ('/File/Exit',        None, self.top.quit, 0, ''),
            ('/Edit',             None, None, 0, '<Branch>'),
            ('/Edit/Find',        None, self.findCB, 1, ''),
            ('/Edit/Find again',  None, self.findCB, 2, ''),
            ('/View',             None, None, 0, '<Branch>'),
            ('/View/All',         None, self.viewCB, V_ALL    , '<CheckItem>'),
            ('/View/Installed',   None, self.viewCB, V_INST   , '<CheckItem>'),
            ('/View/Uninstalled', None, self.viewCB, V_NOTINST, '<CheckItem>'),
	    ('/Package',          None, None, 0, '<Branch>'),
	    ('/Package/Install',  None, self.packageCB, IPKG_INSTALL, ''),
            ('/Package/Remove',   None, self.packageCB, IPKG_REMOVE, ''),
            ('/Package/Upgrade',  None, self.packageCB, IPKG_UPGRADE, '')
            ])
        self.menubar = self.itemf.get_widget('<main>')
        self.box.pack_start(self.menubar, fill=gtk.FALSE, expand=gtk.FALSE)
        self.menubar.show()
        
    def init_pkg_clist(self):
        self.scrollwin = gtk.GtkScrolledWindow()
        self.vpane.pack1( self.scrollwin, gtk.TRUE, gtk.TRUE)
        self.clist = gtk.GtkCList(len(COLS.keys()))
        self.clist.column_titles_show()
        x = 0
        for field_name in COLS.keys():
            self.clist.set_column_title(x,field_name)
            self.clist.set_column_width(x,COLS[field_name]);
            x = x + 1

        self.clist.row_is_visible(1)
        self.clist.connect("select_row" , self.selectRowCB)
        self.clist.connect("click_column", self.selectColumnCB)

        self.scrollwin.add(self.clist)
        self.scrollwin.show()
        self.clist.show()
        
    def init_info(self):
        self.info = gtk.GtkScrolledWindow()
        self.gtkText = gtk.GtkText()
        self.gtkText.set_word_wrap( gtk.TRUE )
        self.info.add(self.gtkText)
        self.info.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        self.vpane.pack2( self.info, gtk.TRUE, gtk.TRUE)
        self.info.show_all()    

    def viewCB(self,action,widget):
        if self.checkItemCalled == 1:   ## Make sure set_active does not 
            return                      ## recall the callback
        self.checkItemCalled = 1;

        ## More nasty workarounds for ItemFactory Radiobutton problems
        for path in [ '/View/All', '/View/Installed', '/View/Uninstalled' ]:
            if widget != self.itemf.get_widget( path ):
                self.itemf.get_widget( path ).set_active(gtk.FALSE)
        self.checkItemCalled = 0;
        
        self.pkgs.update_view(action)
        self.update_pkg_list()

    def findCB(self,action,widget):
        if action == 1:
            MsgBox(self,"Enter Search String",2).show()
        elif action == 2 and self.searchStr != '':
            self.doFind()

    def packageCB(self,action,widget):
        if not self.selected and action != IPKG_UPDATE  and action != IPKG_UPGRADE_ALL:
            MsgBox(self.top,"No package selected!").show()
            return
        ext_cmd = 'rxvt -e '
        xtra_swt = ''
        if not INTERACTIVE:
            ext_cmd = ''
            xtra_swt ='  -force-defaults'
        
        if action == IPKG_INSTALL:     ## TODO: Should check for ipkg returning error ?
            os.system(ext_cmd + IPKG_BIN + xtra_swt + " install " + self.selected )
        elif action == IPKG_REMOVE:
            os.system(ext_cmd + IPKG_BIN + xtra_swt + " remove " + self.selected )
        elif action == IPKG_UPDATE:
            os.system(ext_cmd + IPKG_BIN + xtra_swt + " update ")
            self.pkgs.get_available();
        elif action == IPKG_UPGRADE:
            os.system(ext_cmd + IPKG_BIN + xtra_swt + " upgrade " + self.selected )
        elif action == IPKG_UPGRADE_ALL:
            os.system(ext_cmd + IPKG_BIN + xtra_swt + " upgrade " )

        if not INTERACTIVE:
            MsgBox(self.top,"Command Completed").show()
            
        self.pkgs.get_installed();
        self.update_pkg_list()

    def selectColumnCB(self, *args):
        tosort = int(args[1])
        self.clist.clear()
        self.pkgs.sort(COLS.keys()[tosort])
        self.update_pkg_list()

    # TODO: Make class methods of all the nasty pkg lookupup
    #       stuff in the methods below
            
    def selectRowCB(self, *args):
        self.selected = self.pkgs.view[int(args[1])][FLD_LU["Package"]]
        self.gtkText.set_point(self.gtkText.get_length())
        self.gtkText.backward_delete(self.gtkText.get_length())
        self.gtkText.set_point(0)
        self.gtkText.insert(None,None,None,
                            self.pkgs.view[int(args[1])][FLD_LU["Description"]])
        extra = '\n\n'
        for fld in FLD_LU.keys():
            if not COLS.has_key(fld) and fld != 'Description':
                if self.pkgs.view[int(args[1])][FLD_LU[fld]]:
                    extra = extra + fld + ": " + self.pkgs.view[int(args[1])][FLD_LU[fld]] + '\n'
        self.gtkText.insert(None,None,None, extra)
            
        adj = self.info.get_vadjustment() 
        adj.set_value(0.0) 
        self.gtkText.set_point(0)

    def update_pkg_list(self):
        rowidx = 0
        self.clist.clear()
#       icol  = self.cmap.alloc(51400,51400,51400);
        icol  = self.cmap.alloc(UNINSTALLED_COLOR);
#       incol = self.cmap.alloc(26200,26200,26200);
        incol = self.cmap.alloc(TO_UPGRADE_COLOR);
        for p in self.pkgs.view:
            cols = []
            for f in COLS.keys():
                cols.append(str(p[FLD_LU[f]]))
            self.clist.append( cols )    
            if self.pkgs.is_installed(p[FLD_LU["Package"]]):
                if self.pkgs.get_installed_version(p[FLD_LU["Package"]]) != p[FLD_LU["Version"]]:
                    self.clist.set_background(rowidx, incol ) ## color for installed but newer available
                else:
                    self.clist.set_background(rowidx, icol ) ## color for installed package
            rowidx = rowidx + 1

    def mainloop(self):
        self.top.show()
        gtk.mainloop()

def load_rc():
    global COLS, UNINSTALLED_COLOR, TO_UPGRADE_COLOR,INTERACTIVE
    try:
        rcfile = open(os.environ['HOME'] + "/.pikpak",'r')
    except:
        # write a default file here
        try:
            print "Atempting to create %s" % os.environ['HOME'] + "/.pikpak"
            rcfile = open(os.environ['HOME'] + "/.pikpak",'w')
        except:
            print "ERROR: could not write %s" % os.environ['HOME'] + "/.pikpak"
            sys.exit(0)

        rcfile.writelines( 
            [ '## This is the deualt pikpak configuration file\n',
            '\n',
            '## Define the ipkg attributes to show as columns\n',
            '## Togeather with there widths in pixels\n',
            '\n',
            'COLUMNS = Package:100,Version:50,Size:50,Section:50,Priority:50\n',
            '\n',
            '## Define the colors for upradable and uninstalled packages\n'
            '## See /usr/X11R6/lib/x11/rgb.txt for valid definitions\n',
            '\n',
            'UNINSTALLED_COLOR = dark red\n',
            'TO_UPGRADE_COLOR  = red     \n',
            '\n',
            '## Do you want pikpak to launch ipkg interactively or not\n',
            '## ie, launch a term with ipkg running in it\n',
            '\n',
            'IPKG_INTERACTIVE = YES\n'
            ] )
        rcfile.close()
        return
            
    line = rcfile.readline()
    while line:
        if line[0] == '#' or line[0] == '\n':
            line = rcfile.readline()
            continue
        key,val = split(line,'=')
        key = strip(key)
        val = strip(val)
        if key == 'COLUMNS':
            COLS = {}
            cols = split(val,',')
            if not cols: cols = val ## fix for only 1 column
            for pair in cols:
                n,w = split(pair,':')
                COLS[n] = int(w)
        elif key == 'UNINSTALLED_COLOR':
            UNINSTALLED_COLOR = val
        elif key == 'TO_UPGRADE_COLOR':
            TO_UPGRADE_COLOR  = val
        elif key == 'IPKG_INTERACTIVE':
            if val == 'YES':
                INTERACTIVE = 1
            else:
                INTERACTIVE = 0
        line = rcfile.readline()

def main():
    load_rc()
    s = Pikpak()
    ##s.__init__(self)
    s.mainloop()
if __name__ == "__main__":
    main()

















